/*--------------------------------------------------------------------------
 * Copyright (C) 1992-1998 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 * TMEXLIB.C - This library module contains the interrupt all TMEX 
 *             functions.
 *
 *             The global variable 'DOWInt' must be set 'DOW' interrupt
 *             and the global variable 'TMEXInt' must be set to the
 *             'TMEX' interrupt.  The routine 'FindInts' can
 *             be used to find the correct interrupt vectors.  If the
 *             vectors are not set correctly before calling the interrupt
 *             routines then the PC may need to be rebooted.
 *         
 *
 *  Compiler: Borland C++ 3.1 or Microsoft C 8.0
 *  Version: 3.10
 *
 *  Revision History:
 *
 *     1.00 => 1.10   1) Corrected output in TMNext().
 *
 *                    2) Add functions TMFirstAlarm(), TMNextAlarm(),
 *                       TMReadPacket(), TMWritePacket(), TMBlockIO(),
 *                       TMReNameFile(), TMStrongAccess(), and
 *                       TMStrongAlarmAccess().
 *
 *     1.10 => 2.00   1) Change format of interrupt calling so this library
 *                       can be compiled with Borland or Microsoft C.
 *        
 *                    2) Add functions TMProgramPulse, TMFamilySearchSetup,
 *                       TMSkipFamily, TMExtendedReadPage, TMProgramByte,
 *                       TMChangeDirectory, TMDirectoryMR, 
 *                       TMCreateProgramJob, and TMDoProgramJob.
 *                     
 *                    3) Change the arguments of TMFirstFile, TMNextFile,
 *                       TMOpenFile, TMCreateFile, TMDeleteFile, TMAttribute,
 *                       and TMRenameFile to use the FileEntry structure.
 *
 *                    4) Changed output in HARDWARE_SPECIFIC, NETWORK, and
 *                       TRANSPORT layers to return negative error codes.             
 *
 *     2.00 => 2.10   1) Add functions TMWriteAddFile and TMTerminateAddFile.  
 *
 *     2.10 => 3.00   1) Add functions TMAutoOverDrive, TMOverAccess, 
 *                       TMOneWireLevel and TMOneWireCom.
 *     
 *     3.00 => 3.10   1) Add function TMBlockStream.
 *
 *     3.10 => 3.11   1) Add functions TMSearch, TMGetFamilySpec, TMProgramBlock,
 *                       TMCRC, TMGetAdapterSpec, and TMReadDefaultPort. 
 */

#include "ibtmexcd.h"

/* Function Prototypes */
/*---FILE_OPERATIONS Level Functions---*/
short far pascal TMFirstFile(short, FileEntry far *);
short far pascal TMNextFile(short, FileEntry far *);
short far pascal TMOpenFile(short, FileEntry far *);
short far pascal TMCreateFile(short, short far *, FileEntry far *);
short far pascal TMCloseFile(short, short);
short far pascal TMReadFile(short, short, uchar far *, short);
short far pascal TMWriteFile(short, short, uchar far *, short);
short far pascal TMDeleteFile(short, FileEntry far *);
short far pascal TMFormat(short);
short far pascal TMAttribute(short, short, FileEntry far *);
short far pascal TMReNameFile(short, short, FileEntry far *);  
short far pascal TMChangeDirectory(short, short, DirectoryPath far *); 
short far pascal TMDirectoryMR(short, short, FileEntry far *);      
short far pascal TMCreateProgramJob(short);
short far pascal TMDoProgramJob(short);
short far pascal TMTerminateAddFile(short, FileEntry far *);  /* (2.10) */
short far pascal TMWriteAddFile(short, short, short, short, uchar far *, short);  /* (2.10) */
short far pascal TMGetFamilySpec(short, Specification far *); /* (3.11) */
/*---NETWORK Level Functions---*/
short far pascal TMFirst(short);
short far pascal TMNext(short);
short far pascal TMAccess(short);
short far pascal TMStrongAccess(short);      
short far pascal TMStrongAlarmAccess(short); 
short far pascal TMOverAccess(short); /* (3.00) */
uchar far * far pascal TMRomData(short);
short far pascal TMRom(short, short far *);
short far pascal TMFirstAlarm(short);        
short far pascal TMNextAlarm(short);         
short far pascal TMFamilySearchSetup(short, short); 
short far pascal TMSkipFamily(short);
short far pascal TMAutoOverDrive(short, short); /* (3.00) */
short far pascal TMSearch(short, short, short, short); /* (3.11) */
/*---TRANSPORT Level Functions---*/
short far pascal TMReadPacket(short, short, uchar far *, short);
short far pascal TMWritePacket(short, short, uchar far *, short);
short far pascal TMBlockIO(short, uchar far *, short);     
short far pascal TMExtendedReadPage(short, short, uchar far *, short);
short far pascal TMProgramByte(short, short, short, short, short far *, short);
short far pascal TMProgramBlock(short, uchar far *, short, short, short far *);  /* 3.11 */
long  far pascal TMCRC(short, uchar far *, ushort, short);  /* 3.11 */
/*---HARDWARE_SPECIFIC Level Functions---*/
short far pascal TMSetup(short);
short far pascal TMTouchByte(short, short);
short far pascal TMTouchReset(short);
short far pascal TMTouchBit(short, short);
short far pascal TMProgramPulse(short);
short far pascal TMClose(short);
short far pascal TMOneWireLevel(short, short, short, short); /* 3.00 */
short far pascal TMOneWireCom(short, short, short); /* 3.00 */
short far pascal TMBlockStream(short, uchar far *, short);  /* 3.10 */
short far pascal TMGetAdapterSpec(short, Specification far *);  /* 3.11 */
short far pascal TMReadDefaultPort(short far *, short far *);  /* 3.11 */
/*---Utilities to find interrupts ---*/
short far pascal FindInts(short far *, short far *);
short far pascal GetVectID(short, char far *);

/* interrupt number for the 1-Wire functions */
extern short DOWInt;
extern short TMEXInt;

/* structure to hold register values when calling interrupts */
static union REGS inreg,outreg;  
static struct SREGS sreg;        


/*--------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 * FILE_OPERATIONS - These API calls do file and directory operations.  A
 *                   valid ROM in the internal 8 byte buffer is required 
 *                   before making any of these API calls.  See the NETWORK
 *                   API calls for getting a valid ROM.
 *
 * The FILE_OPERATIONS error codes are:
 *
 *   -1  : no device found on 1-Wire 
 *   -2  : wrong type of device for requested operation 
 *   -3  : file/directory read error 
 *   -4  : buffer provided is smaller then read file 
 *   -5  : there are no more file handles left 
 *   -6  : file specified is not in the current directory 
 *   -7  : file already exists with name provided 
 *   -8  : given file handle is not assigned a file 
 *   -9  : trying to read a write file handle 
 *   -10 : not enough room on device to write 
 *   -11 : error during write, part may have expired 
 *   -12 : trying to write a read file handle 
 *   -13 : function not supported 
 *   -14 : illegal file/directory name 
 *   -15 : trying to delete a read only file 
 *   -16 : file handle does not exist 
 *   -17 : error in 1-Wire port, port may not be setup using TMSetup     
 *   -18 : directory given is invalid 
 *   -19 : directory must be empty to delete it 
 *   -20 : maximum depth of 10 sub-dirs is reached 
 *   -21 : there is no program job started 
 *   -22 : device is written in a way that can not be changed 
 *   -23 : there are non-program parts on the 1-Wire during an attempt 
 *         to program           
 *   -24 : the addfile selected has already been terminated  (2.10) 
 *
 */


/*--------------------------------------------------------------------------
 * This API call finds the first file on 1-Wire 'port' in the current
 * directory and returns a pointer to a structure containing the filename 
 * and other information.  The data in the structure is copied over to 
 * the buffer 'fentry'.  The structure is as follows:
 *
 *   typedef struct      / typedef structure sent back by TMEX routines /
 *   {
 *       uchar name[4];       / 4 digit DOS type file characters /
 *       uchar extension;     / extension 0 - 99, 127 /
 *       uchar startpage;     / file start page /
 *       uchar numpages;      / length of file in number of pages /
 *       uchar attrib;        / 1 if file is readonly or 0 if not /
 *       uchar bitmap[32];    / current bitmap of the device /
 *   } FileEntry;
 *
 * This function returns:
 *                        >0 : file found, data in buffer 'fentry'
 *                         0 : device has an empty directory 
 *                             (bitmap of 'fentry' set anyway)
 *                        <0 : FILE_OPERATIONS error
 *
 */
short far pascal TMFirstFile(short port, FileEntry far *fentry)
{
     FileEntry far *fp;

     inreg.x.ax = ((port << 4) | 0x01) << 8;

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     /* get the pointer to the file name */
     fp = (FileEntry far *)MK_FP(sreg.es,outreg.x.bx);

     /* copy the file over to this buffer */
     *fentry = *fp;

     return outreg.x.ax & 0xFF;  /* return number of file entries */
}


/*--------------------------------------------------------------------------
 * This API call finds the next file on 1-Wire 'port' in the current 
 * directory and returns a pointer to a structure containing the filename 
 * and other information.  The data in the structure is copied over 
 * to the structure 'fentry'.  The structure is described in 'TMFirstFile'.
 *
 * This function returns:
 *                         1 : file found, data in 'fentry'
 *                         0 : no more files in the directory
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMNextFile(short port, FileEntry far *fentry)
{
     FileEntry far *fp;

     inreg.x.ax = ((port << 4) | 0x02) << 8;

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     /* get the pointer to the file name */
     fp = (FileEntry far *)MK_FP(sreg.es,outreg.x.bx);

     /* copy the file over to this buffer */
     *fentry = *fp;

     return outreg.x.ax & 0xFF;  /* return success or fail */
}


/*--------------------------------------------------------------------------
 * This API call opens a file for a future call to TMReadFile. The filename 
 * to open is specified in 'fentry' which is in the structure FileEntry as 
 * described in TMFirstFile.  Only the 'name' and 'extension' portions of
 * FileEntry need to be set before calling TMOpenFile. The file must be in 
 * the current directory on the current device on the 1-Wire 'port'.  This 
 * API returns a file handle to be used when calling TMReadFile. 
 *
 * This function returns:
 *                       >=0 : file found, and this is the file handle
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMOpenFile(short port, FileEntry far *fentry)
{
     inreg.x.ax = ((port << 4) | 0x03) << 8;
     inreg.x.bx = FP_OFF(fentry);          /* set offset and segment */
     sreg.es = FP_SEG(fentry);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.ax & 0xFF;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API function creates a new file name in the current directory on
 * the current device on the 1-Wire 'port'.  The new file name is provided 
 * in the FileEntry structure 'fentry'.  Only the 'name' and 'extension' 
 * portions of FileEntry need to be set before calling TMCreateFile.  Repeat 
 * file names are not permitted. This API returns a file handle >= 0. The 
 * API call also returns the estimated maximum number of bytes available 
 * for this new file 'maxwrite'.  The file name must be left justified
 * padded with spaces.  The valid file name characters are 
 * (ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!#$%&'-@^_`{}~).  The valid value 
 * of the extension is 0-99 decimal. 
 *
 * This function returns:
 *                       >=0 : file created, and this is the file handle
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMCreateFile(short port, short far *maxwrite, FileEntry far *fentry)
{
     inreg.x.ax = ((port << 4) | 0x04) << 8;
     inreg.x.bx = FP_OFF(fentry);          /* set offset and segment */
     sreg.es = FP_SEG(fentry);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     /* set the maximum number of bytes available for a file */
     *maxwrite = outreg.x.cx;

     return outreg.x.ax & 0xFF;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call closes the file specified by the file handle 'file_handle'. 
 * All files that have been opened for reading with TMOpenFile or created 
 * for writing with TMCreateFile must be closed after use. 
 *
 * This function returns:
 *                         1 : file closed
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMCloseFile(short port, short file_handle)
{
     inreg.x.ax = (((port << 4) | 0x05) << 8) | (file_handle & 0x0F);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call reads from the file specified by the file handle number
 * 'file_handle' and copies it into the buffer 'read_buffer', not to exceed
 * 'max_read' bytes.  'file_handle' must be set as a result of a call to
 * TMOpenFile.
 *
 * This function returns:
 *                       >=0 : file read, and this is the number of bytes
 *                             read
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMReadFile(short port, short file_handle, uchar far *read_buffer,
                                                              short max_read)
{
     inreg.x.ax = (((port << 4) | 0x06) << 8) | (file_handle & 0x0F);
     inreg.x.cx = max_read;                  /* set characters to read */
     inreg.x.bx = FP_OFF(read_buffer);          /* set offset and segment */
     sreg.es = FP_SEG(read_buffer);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.cx;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * This API call writes the file specified by the file handle 'file_handle' 
 * with the data from the buffer 'write_buffer', containing 'num_write' bytes. 
 * 'file_handle' must be set as a result of a call to TMCreateFile.
 *
 * This function returns:
 *                       >=0 : file written, and this is the number of bytes
 *                             written.
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMWriteFile(short port, short file_handle, uchar far *write_buffer,
                                                              short num_write)
{
     inreg.x.ax = (((port << 4) | 0x07) << 8) | (file_handle & 0x0F);
     inreg.x.cx = num_write;                  /* set characters to write */
     inreg.x.bx = FP_OFF(write_buffer);          /* set offset and segment */
     sreg.es = FP_SEG(write_buffer);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.cx;  /* return successful number chars written */
}


/*--------------------------------------------------------------------------
 * This API call deletes the file specified in the FileEntry structure
 * 'fentry'.  Only the 'name' and 'extension' portions of 'fentry' need
 * be set before calling TMDeleteFile.  The file must be in the current
 * directory on the current device on the 1-Wire port 'port'.
 *
 * This function returns:
 *                         1 : file deleted
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMDeleteFile(short port, FileEntry far *fentry)
{
     inreg.x.ax = ((port << 4) | 0x08) << 8;
     inreg.x.bx = FP_OFF(fentry);          /* set offset and segment */
     sreg.es = FP_SEG(fentry);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call writes an empty directory into the current device on the 
 * 1-Wire port 'port'.
 *
 * This function returns:
 *                         1 : device formatted
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMFormat(short port)
{
     inreg.x.ax = ((port << 4) | 0x09) << 8;

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call changes the attributes of the file or directory 'fentry' 
 * to the attributes 'attribute'.  Only the 'name' and 'extension'
 *  portions of 'fentry' need be set before calling TMAttribute.  
 * 'attribute' contains bits that are set to apply an attribute and 
 * cleared to remove an attribute.  'bit 0' represents the least 
 * significant bit in 'attribute'.  Here are the attributes for TMEX 2.00:
 *
 *   bit 0 : set to make file read-only, for files only
 *   bit 1 : set to make directory hidden, for directories only 
 *   bit 2-7 : reserved for future attributes
 *
 * Note that when specifying a directory the 'extension' is 127.
 *
 * This function returns:
 *                         1 : file attribute changed to 'attr'
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMAttribute(short port, short attribute, FileEntry far *fentry)
{
     inreg.x.ax = (((port << 4) | 0x0A) << 8) | (attribute & 0xFF);
     inreg.x.bx = FP_OFF(fentry);          /* set offset and segment */
     sreg.es = FP_SEG(fentry);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call changes the name of a previously opened file specified 
 * by the 'file_handle'.  The new name is given in the FileEntry structure 
 * 'fentry'.  Only the 'name' and 'extension' portions of 'fentry' need
 * be set before calling TMReNameFile.  'file_handle' must be set as a 
 * result of a call to TMOpenFile.
 *
 * This function returns:
 *                         1 : file renamed
 *                        <0 : FILE_OPERATIONS error  
 */
short far pascal TMReNameFile(short port, short file_handle, FileEntry far *fentry)
{
     inreg.x.ax = (((port << 4) | 0x0B) << 8) | (file_handle & 0x0F);
     inreg.x.bx = FP_OFF(fentry);          /* set offset and segment */
     sreg.es = FP_SEG(fentry);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call sets or reads the current working directory.  There is an
 * 'operation' flag that indicates if the call is to set or read the current
 * directory. Here are the values of 'operation':
 *
 *   0 : current directory is set
 *   1 : current directory is read
 *
 * The value in 'cd_buf' is a DirectoryPath structure that either contains 
 * the directory to be set or the current directory read depending on the 
 * 'operation' flag.  'cd_buf' must be the following DirectoryPath structure. 
 *
 *   typedef struct  
 *   {
 *       unsigned char NumEntries;  / number of entries in structure /
 *       char Ref;                  / directory reference character /
 *       char Entries[10][4];       / entry items /
 *   } DirectoryPath;
 *
 * The DirectoryPath structure is modeled after the manner working 
 * directory paths are written and read using the 'cd' DOS command.
 *
 * This function returns:
 *                         1 : operation successful
 *                        <0 : FILE_OPERATIONS error  
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMChangeDirectory(short port, short operation, 
                                                 DirectoryPath far *cd_buf)
{
     inreg.x.ax = (((port << 4) | 0x0C) << 8) | (operation & 0xFF);
     inreg.x.bx = FP_OFF(cd_buf);          /* set offset and segment */
     sreg.es = FP_SEG(cd_buf);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}

/*--------------------------------------------------------------------------
 * This API call Makes or Removes a sub-directory from the current 
 * directory.  There is an 'operation' flag that indicates if the call is 
 * to make or remove a sub-directory. Here are the values of 'operation':
 *
 *   0 : make sub-directory specified by FileEntry 'dir_buf'
 *   1 : remove sub-directory specified by FileEntry 'dir_buf'
 *
 * Only the 'name' field of the FileEntry structure need be filled in 
 * before calling TMDirectoryMR.  Note that a sub-directory is created empty
 * and only an empty sub-directory can be removed.
 *
 * This function returns:
 *                         1 : operation successful
 *                        <0 : FILE_OPERATIONS error  
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMDirectoryMR(short port, short operation, FileEntry far *cd_buf)
{
     inreg.x.ax = (((port << 4) | 0x0D) << 8) | (operation & 0xFF);
     inreg.x.bx = FP_OFF(cd_buf);          /* set offset and segment */
     sreg.es = FP_SEG(cd_buf);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}
 
                               
/*--------------------------------------------------------------------------
 * This API call starts a program job with the current EPROM device.  This 
 * function must be called immediately before any TMEX function that writes
 * to an EPROM device.  The writes in the TMEX function are not actually 
 * done, they are just logged in the program job data space.   After the 
 * TMEX function has been completed successfully, then another specialty 
 * function TMDoProgramJob must be called to carry out the EPROM programming.  
 * EPROMs are treated differently by TMEX because any write to an EPROM is 
 * destructive, meaning it uses up device memory.  In NV-RAM devices, if 
 * an operation is unsuccessful then the function could then be just called
 * again with no ill effects.  If EPROMs were regarded in this way then some
 * or all of the memory space could be lost. The second specialty API call
 * TMDoProgramJob is called again and again by the application program
 * until all of the jobs are completed.  This puts the onus on the
 * application programmer to make sure that the EPROM device does not go 
 * away in mid write and waist memory space.  For instance if  after doing 
 * a TMCreateProgramJob and TMWriteFile successfully, the TMDoProgramJob 
 * comes back saying that device has left the 1-Wire then the application 
 * should put up some message saying that the device needs to come back and
 * call TMDoProgramJob again.  The data space needed to write EPROMs is
 * located in an internal buffer in the DOS TSR.   There can be only one 
 * program job at a time.  If after TMCreateProgramJob is called, the 
 * internal ROM buffer is changed then the program job is invalidated. 
 *
 * This function returns:
 *                         1 : Program job has been started
 *                        <0 : FILE_OPERATIONS error  
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMCreateProgramJob(short port)                               
{
     inreg.x.ax = (((port << 4) | 0x0E) << 8) | 0x01;

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}
                           
                           
/*--------------------------------------------------------------------------
 * This API call writes all of the data that has been written to the EPROM 
 * device since the TMCreateProgramJob function was called.  The write jobs 
 * are done one at time until complete.  This function marks off the jobs 
 * it has done so that if writing can not continue because the device has
 * lost contact it can begin where it left off when it is called again.  If
 * this function fails and is not called again and allowed to complete, then
 * the target device may be left in an unknown state.  EPROMs are treated
 * differently by TMEX because any write to an EPROM is destructive, meaning
 * it uses up device memory. In NV-RAM devices, if an operation is
 * unsuccessful then the function could then be just called again with no
 * ill effects.  If EPROMs were regarded in this way then some or all of
 * the memory space could be lost.  The TMDoProgramJob function is called
 * again and again by the application program until all of the jobs are
 * completed.  This puts the onus on the application programmer to make
 * sure that the EPROM device does not go away in mid write and waste memory
 * space and leave the device in an unknown state. If the ROM is changed
 * at any time before TMDoProgramJob is finished then the program job
 * becomes invalid.
 *
 * This function returns:
 *                         1 : Program job is completed
 *                        <0 : FILE_OPERATIONS error  
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMDoProgramJob(short port)                               
{
     inreg.x.ax = (((port << 4) | 0x0E) << 8) | 0x02;

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 * This API call writes the add file specified by the file handle 'file_handle' 
 * with the data from the buffer 'add_buffer', containing 'num_write' bytes. 
 * 'file_handle' must be set as a result of a call to TMCreateFile or 
 * TMOpenFile.  The 'operation' flag is '1' for appending the data to the 
 * end of the file or '0' for writing at an 'offset' from the beginning of
 * the file.
 *
 * This function returns:
 *                       >=0 : file written, and this is the number of bytes
 *                             written.
 *                        <0 : FILE_OPERATIONS error  
 *
 * (2.10) This function is new for version 2.10.
 */
short far pascal TMWriteAddFile(short port, short file_handle, short operation, short offset,
                              uchar far *add_buffer, short num_write)
{
     inreg.x.ax = (((port << 4) | 0x0E) << 8) | 0x03;
     inreg.x.cx = (file_handle << 14) | num_write; 
     inreg.x.dx = (operation << 14) | offset;
     inreg.x.bx = FP_OFF(add_buffer);          /* set offset and segment */
     sreg.es = FP_SEG(add_buffer);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */
     else
        return outreg.x.cx;  /* return successful number chars written */
}


/*--------------------------------------------------------------------------
 * This API call terminates an 'add' file on an EPROM Touch Memory.  It adds
 * the length and CRC bytes and terminates the last file page with a 0 
 * continuation pointer.
 *
 * This function returns:
 *                         1 : file terminated
 *                        <0 : FILE_OPERATIONS error  
 *
 * (2.10) This function is new for version 2.10.
 */
short far pascal TMTerminateAddFile(short port, FileEntry far *fentry)
{
     inreg.x.ax = (((port << 4) | 0x0E) << 8) | 0x04;
     inreg.x.bx = FP_OFF(fentry);          /* set offset and segment */
     sreg.es = FP_SEG(fentry);

     int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return 1;  /* return success */
}


/*--------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 * NETWORK - These API calls are provided to support the mult-drop 
 *           capability of the 1-Wire protocal.  They allow the calling
 *           program to locate and individually communicate with one of 
 *           many Touch Memory devices connected in parallel to the 1-Wire
 *           bus.
 * 
 * The return error codes for the NETWORK calls are:
 *
 *   -1  : port not initialize with TMSetup
 *   -2  : port does not exist
 *   -3  : API call not supported
 */


/*--------------------------------------------------------------------------
 * This API call finds the first multi-drop device on the 1-Wire 'port'.  
 * The return values are:
 *                                                    
 *    0  : device not found
 *    1  : first device found on 1-Wire 'port'
 *    <0 : NETWORK error 
 */
short far pascal TMFirst(short port)
{
     inreg.x.ax = ((port << 4) | 0x04) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return outreg.x.ax & 0x00FF;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * This API call looks for the next multi-drop device on the 1-Wire 'port'.  
 * The return values are:
 *                                                    
 *     0 : no more 'next' device on 1-Wire 'port' (end of list)
 *     1 : next device found on 1-Wire 'port'
 *    <0 : NETWORK error 
 */
short far pascal TMNext(short port)
{
     inreg.x.ax = ((port << 4) | 0x05) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return outreg.x.ax & 0x00FF;  
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * This API call sets up the current device for a device specific command 
 * such as a read memory commmand.  It does this by sending the MATCH 
 * command to the 1-Wire followed by the ROM number.  Note that the ROM 
 * must already be set in the internal 8 byte buffer before calling this 
 * function.  The ROM can be set by calling API functions such as 'TMFirst' 
 * or 'TMRom'.
 * The return values are:
 *                                                    
 *     0 : no presence detect on 1-Wire 'port'
 *     1 : a Touch Memory is on 1-Wire 'port' and MATCH sent. 
 *    <0 : NETWORK error 
 */
short far pascal TMAccess(short port)
{
     short i;

     inreg.x.ax = ((port << 4) | 0x06) << 8;

     for (i = 0; i < 5; i++)
     {
          int86x(DOWInt,&inreg,&outreg,&sreg);       /* call the interrupt */
          if (outreg.x.cflag)  /* check on carry */
             return -(short)(outreg.x.ax & 0x00FF);        /* carry is set */

          if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x03 )
             continue; /* if shorted, try again */
          else
             return outreg.x.ax & 0x00FF;
     }

     return 0;   /* failed after 5 shorts */
}


/*--------------------------------------------------------------------------
 * This API call sets up the current device for a device specific command 
 * such as a read memory commmand and verifies the device is on the 1-Wire
 * port.  It does this by using the SEARCH ROM command to select that device
 * and at the same time verify that it is connected to the 1-Wire. Note 
 * that the ROM must already be set in the internal 8 byte buffer before 
 * calling this function.  The ROM can be set by calling API functions such 
 * as 'TMFirst' or 'TMRom'.
 * The return values are:
 *                                                    
 *     0 : device in ROM buffer is not on 1-Wire port
 *     1 : device in ROM buffer on 1-Wire and it is selected
 *    <0 : NETWORK error 
 */
short far pascal TMStrongAccess(short port)
{
     short i;

     inreg.x.ax = (((port << 4) | 0x06) << 8) | 0xF0;

     for (i = 0; i < 5; i++)
     {
          int86x(DOWInt,&inreg,&outreg,&sreg);       /* call the interrupt */
          if (outreg.x.cflag)  /* check on carry */
             return -(short)(outreg.x.ax & 0x00FF);        /* carry is set */

          if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x03 )
             continue; /* if shorted, try again */
          else
             return outreg.x.ax & 0x00FF;
     }

     return 0;   /* failed after 5 shorts */
}


/*--------------------------------------------------------------------------
 * This API call sets up the current device for a device specific command 
 * such as a read memory commmand and verifies the device is on the 1-Wire
 * port and alarming.  It does this by using the SEARCH INTERRUPTER ROM 
 * command to select that device only if it is on the 1-Wire and in an 
 * alarming state.  Note that the ROM must already be set in the internal 
 * 8 byte buffer before calling this function.  The ROM can be set by 
 * calling API functions such as 'TMFirst' or 'TMRom'.
 * The return values are:
 *                                                    
 *     0 : device in ROM buffer is not on 1-Wire port or is not alarming
 *     1 : device in ROM buffer on 1-Wire and alarming and it is selected
 *    <0 : NETWORK error 
 */
short far pascal TMStrongAlarmAccess(short port)
{
     short i;

     inreg.x.ax = (((port << 4) | 0x06) << 8) | 0xEC;

     for (i = 0; i < 5; i++)
     {
          int86x(DOWInt,&inreg,&outreg,&sreg);       /* call the interrupt */
          if (outreg.x.cflag)  /* check on carry */
             return -(short)(outreg.x.ax & 0x00FF);        /* carry is set */

          if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x03 )
             continue; /* if shorted, try again */
          else
             return outreg.x.ax & 0x00FF;
     }

     return 0;   /* failed after 5 shorts */
}


/*--------------------------------------------------------------------------
 * This API call resets the 1-Wire and sends an Overdrive MATCH ROM 
 * command, changes to overdrive speeds, and then sends the
 * ROM code.  If this fails at any time in the process
 * then a 0 is returned, otherwise 1 is returned.
 */
short far pascal TMOverAccess(short port)
{
     short i;

     inreg.x.ax = (((port << 4) | 0x06) << 8) | 0x69;

     for (i = 0; i < 5; i++)
     {
          int86x(DOWInt,&inreg,&outreg,&sreg);       /* call the interrupt */
          if (outreg.x.cflag)  /* check on carry */
             return -(short)(outreg.x.ax & 0x00FF);        /* carry is set */

          if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x03 )
             continue; /* if shorted, try again */
          else
             return outreg.x.ax & 0x00FF;
     }

     return 0;   /* failed after 5 shorts */
}                                  

                                 
/*--------------------------------------------------------------------------
 * This API call returns a pointer to the 8 byte ROM storage area for the 
 * 1-Wire port 'port'.
 */
uchar far * far pascal TMRomData(short port)
{
     inreg.x.ax = ((port << 4) | 0x07) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     return (uchar far *)MK_FP(sreg.es,outreg.x.bx);
}


/*--------------------------------------------------------------------------
 * This API call transfers a ROM data pattern between the internal 8-byte
 * buffer maintained by the DLL and an array 'ROM' of eight integers 
 * declared in the application program. The direction of data transfer 
 * is specified by the value of the first integer in the array. If the 
 * first integer is zero, then the eight bytes from the internal buffer 
 * are transferred into the eight integer variables of the integer array. 
 * If the first integer is non-zero, then the least significant bytes of 
 * the eight integers are transferred into the internal eight-byte buffer. 
 * (This function allows an application program to obtain the ROM data of 
 * a device that has been found with the TMFirst or TMNext functions. It 
 * also allows the application program to specify the ROM data of a
 * specific device to be addressed with a multi-drop API call such as
 * TMFirstFile.)  This function returns 1. 
 */
short far pascal TMRom(short port, short far *ROM)
{
     uchar far *CurROM;
     short i;

     /* get a pointer to the current rom */
     CurROM = TMRomData(port);

     if (ROM[0] == 0)  /* just read the ROM number  */
     {
          for (i = 0; i < 8; i++)
              ROM[i] = CurROM[i];
     }
     else  /* copy new rom to current ROM */
     {
          for (i = 0; i < 8; i++)
              CurROM[i] = ROM[i];
     }

     return 1;
}


/*--------------------------------------------------------------------------
 * This API call looks for the first alarming multi-drop device on the 
 * 1-Wire 'port'.   The DS1994, DS1920, and DS2407 currently have an alarm
 * condition.  The return values are:
 *                                                    
 *     0 : no alarming device found
 *     1 : first alarming device found on 1-Wire 'port'
 *    <0 : NETWORK error 
 */
short far pascal TMFirstAlarm(short port)
{
     inreg.x.ax = ((port << 4) | 0x08) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return outreg.x.ax & 0x00FF;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * This API call looks for the next alarming multi-drop device on the 
 * 1-Wire 'port'.  The DS1994, DS1920, and DS2407 currently have an alarm
 * condition.  The return values are:
 *                                                      
 *     0 : no more 'next alarming' device on 1-Wire 'port' (end of list)
 *     1 : next alarming device found on 1-Wire 'port'
 *    <0 : NETWORK error 
 */
short far pascal TMNextAlarm(short port)
{
     inreg.x.ax = ((port << 4) | 0x09) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return outreg.x.ax & 0x00FF;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * This API call sets the search algorithm to look for a particular 
 * family of Touch Memory during the next search on the 1-Wire 'port'.  
 * This function does not effect the 1-Wire in any way, it just sets the 
 * search algorithm up for the next time a search function is called (i.e. 
 * TMNext or TMNextAlarm).  This function sets the search to find a certain 
 * type on the next call only if there is one available.  If there is not 
 * one available, a search function may still find a device of a different 
 * type.  The normal sequence to search for a particulay family type is as 
 * follows.
 *     1) Call TMFamilySearchSetup with the desired family type
 *     2) Call TMNext or TMNextAlarm
 *     3) if 2) is successful then check the family code using TMRomData
 *        to see if the correct type was found
 *     4) if 2) is not successful or the incorrect family code was found then
 *        no devices of that type were on the 1-Wire at the time of the
 *        search             
 *
 * The return values are:             
 *
 *     1 : search algorithm set up for next search to find 'family_type'
 *    <0 : NETWORK error 
 */
short far pascal TMFamilySearchSetup(short port, short family_type)
{
     inreg.x.ax = (((port << 4) | 0x0D) << 8) | 0x01;
     inreg.x.cx = family_type & 0x00FF;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return 1;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}

/*--------------------------------------------------------------------------
 * This API call skips the current family type of Touch Memory that was 
 * found with the last search function.  This function will only work if 
 * there is a search in progress. The normal sequnce to skip a particular 
 * family type is as follows:
 *     1) Call TMFirst or TMFirstAlarm the first pass or TMNext or 
 *         TMNextAlarm in subsequent passes.
 *     2) if step 1) is successful then check the family type using TMRom 
 *         to see if the family code needs to be skipped.
 *     3) call TMSkipFamily if the family code needs to be skipped
 *     4) if step 2) is not successful then no more devices are on the 1-Wire.
 *     5) go back to 1) if not done with search
 *
 * The return values are:             
 *
 *     1 : search algorithm set up for next search to skip current 'family_type'
 *    <0 : NETWORK error 
 */
short far pascal TMSkipFamily(short port)
{
     inreg.x.ax = (((port << 4) | 0x0D) << 8) | 0x02;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return 1;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 * TRANSPORT - These API calls are provided to enable block level data
 *             tranfer to Touch Memory.  This level includes TMReadPacket
 *             and TMWritePacket that read and write CRC verified blocks
 *             of data.
 * 
 * The TRANSPORT error codes are:
 *
 *   -1  : port not initialize with TMSetup
 *   -2  : port does not exist
 *   -3  : API call not supported
 *   -4  : error reading or writing a packet 
 *   -5  : packet larger than provided buffer 
 *   -6  : not enough room for packet on device 
 *   -7  : device not found
 *   -8  : block transfer too long 
 *   -9  : wrong type of device for this function 
 *  -10  : the page being read is redirected 
 *         the redirected page number is in buffer 
 *  -11  : the device is written in a way 
 *         that can not be written to any longer 
 */


/*--------------------------------------------------------------------------
 * This API call reads a Universal Data Packet starting at page 
 * 'start_page'.  The data is placed in the buffer 'read_buffer'.  
 * The maximum number of bytes in the buffer is 'max_read'.
 *
 * This function returns:
 *                       >=0 : file read, and this is the number of bytes
 *                             read
 *                        <0 : TRANSPORT error
 *
 */
short far pascal TMReadPacket(short port, short start_page, uchar far *read_buffer,
                                                              short max_read)
{
     inreg.x.ax = (((port << 4) | 0x0A) << 8) | start_page;
     inreg.x.cx = max_read;                  /* set characters to read */
     inreg.x.bx = FP_OFF(read_buffer);          /* set offset and segment */
     sreg.es = FP_SEG(read_buffer);

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.cx;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * This API call writes a Universal Data Packet starting at page 
 * 'start_page'.  The data is taken from the buffer
 * 'write_buffer'.  The number characters to write is 'num_write'.
 *
 * This function returns:
 *                       >=0 : file written, and this is the number of bytes
 *                             written
 *                        <0 : TRANSPORT error 
 */
short far pascal TMWritePacket(short port, short start_page, uchar far *write_buffer,
                                                             short num_write)
{
     inreg.x.ax = (((port << 4) | 0x0B) << 8) | start_page;
     inreg.x.cx = num_write;                   /* set number to write */
     inreg.x.bx = FP_OFF(write_buffer);        /* set offset and segment */
     sreg.es = FP_SEG(write_buffer);

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.cx;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * This API call transfers a block of 'num_tran' number of bytes to and 
 * from the 1-Wire following a TMTouchReset.  The values returned from
 * the 1-Wire are placed back in the 'tran_buffer'.  Maximum number of 
 * bytes is 1023.
 *
 * This function returns:
 *                       >=0 : data transfered, and this is the number of
 *                             bytes transfered.
 *                        <0 : TRANSPORT error 
 */
short far pascal TMBlockIO(short port, uchar far *tran_buffer, short num_tran)
{
     inreg.x.ax = (((port << 4) | 0x0C) << 8);
     inreg.x.cx = num_tran;                   /* set number to write */
     inreg.x.bx = FP_OFF(tran_buffer);           /* set offset and segment */
     sreg.es = FP_SEG(tran_buffer);

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.cx;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * This API call reads an EXTENDED READ status page or a data page.  Some
 * types of Touch Memory have a self generated CRC so that just a single
 * bit can be changed in a page and still be read with CRC checking. 
 * This function reads the specified status or data page and puts it in 
 * the supplied buffer 'read_buffer' without the CRC bytes.  When reading 
 * a data page from a DS1985 or D1986, the 33rd byte of the provided buffer 
 * will contain the non-inverted redirection byte.  The 'read_buffer' must 
 * be at least 8 bytes long for a status page (page_type = 1) or at least 
 * 33 bytes for a data page (page_type = 0) for the current devices.
 *
 * This function returns:
 *                      >0 : page read success and return number bytes
 *                           (8 or 32 for current devices)
 *                      <0 : TRANSPORT error 
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMExtendedReadPage(short port, short page, uchar far *read_buffer, 
                                                                 short page_type)
{
     inreg.x.ax = (((port << 4) | 0x0D) << 8) | 0x03;  
     inreg.x.cx = ((page_type & 0x01) << 8) | (page & 0x00FF);
     inreg.x.bx = FP_OFF(read_buffer);           /* set offset and segment */
     sreg.es = FP_SEG(read_buffer);

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.ax & 0xFF;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * Call the Transport layer function 'ProgramByte'.
 * Write an byte to the status or regular memory space of a DS1982,
 * DS1985, or DS1986.  The address is in Addr in space page_type (0 Regular,
 * 1 status) with 'Bits' as number of bits per pass (2,4,8).  The 'zeros'
 * variable specifies if the byte to write must be an exact match 
 * (zeros = 0) or if just the zeros specified in the byte to write have 
 * been written (zeros = 1) for success.  It is the job of the caller to 
 * make sure that no other devices are on the line at the same time that 
 * can not handle a programming pulse.
 *
 * This function returns:
 *                       1 : Byte written successfully.
 *                      <0 : TRANSPORT error 
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMProgramByte(short port, short write_byte, short Addr,
                                         short page_type, short far *Bits, short zeros)
{
     inreg.x.ax = (((port << 4) | 0x0D) << 8) | 0x04;  
     inreg.x.cx = Addr;   
     inreg.x.dx = ((zeros & 0x01) << 11) | ((page_type & 0x01) << 10) | 
                  ((*Bits << 6) & 0x0300) | (write_byte & 0x00FF);

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */
     
     /* return the last number of bits per pass */
     *Bits = 0x02 << ((outreg.x.dx >> 8) & 0x03);
     
     return outreg.x.ax & 0xFF; 
}


/*--------------------------------------------------------------------------
 * This API call sets or clears the Auto-Overdrive mode. 
 *
 * Under the following conditions the TMEX driver will automatically 
 * switch to Overdrive:
 *
 * 1. TMAutoOverDrive has be called with Mode = 1
 * 2. The serial ROM number family type supports Overdrive
 * 3. TMAccess is called.  
 * 4. Successful communication to the part is achieved at Overdrive speeds
 *
 * The following conditions will switch the TMEX driver out of Overdrive 
 * if communication is currently in Overdrive due to TMAutoOverDrive:
 *
 * 1. Any search NETWORK API call such as TMFirst or TMNext.  A change to 
 *    the serial ROM number using TMRom. 
 * 2. A failure of TMAccess at Overdrive speeds
 *
 * This function returns:
 *                       >=0 : data transfered, and this is the number of
 *                             bytes transfered.
 *                        <0 : TRANSPORT error 
 */
short far pascal TMAutoOverDrive(short port, short Mode)
{
     inreg.x.ax = (((port << 4) | 0x0D) << 8) | 0x05;  
     inreg.x.cx = Mode & 0x00FF;   

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */
     
     return outreg.x.ax & 0xFF;  /* return result */
}


/*--------------------------------------------------------------------------  
 *--------------------------------------------------------------------------
 * HARDWARE_SPECIFIC - These function are the lowest level functions that
 *                     directly interface to hardware.
 * 
 * Some of the HARDWARE_SPECIFIC functions return negative error codes.
 * They are:
 *
 *   -1  : port not initialize with TMSetup
 *   -2  : port does not exist
 *   -3  : API call not supported
 */
 
/*--------------------------------------------------------------------------
 * This API call verifies the existence of the 1-Wire port 'port' and 
 * returns its state.
 *
 * This function returns:    0 setup failed
 *                           1 setup ok on comport
 *                           2 setup ok but got a short
 *                           3 1-Wire does not exist
 *                           4 TMSetup not supported
 */
short far pascal TMSetup(short port)
{
     inreg.x.ax = (port << 4) << 8;  /* select the port and TMSetup, Clear AL  */

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
     {
          if ((outreg.x.ax & 0x00FF) == 1) /*  port found */
             return 1;
          else if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x03 )  /* shorted */
             return 2;
          else
             return 0;  /* not shorted and failed  */
     }
     else
     {                                     
          /*  1-Wire port does not exist */
          if ((outreg.x.ax & 0x00FF) == 2) 
             return 3;
          else
             return 4;        /* Setup function not supported */
     }
}


/*--------------------------------------------------------------------------
 * This API call resets all of the devices on the 1-Wire port 'port' and 
 * returns the state.
 *
 * This function returns:    1 non alarming presence pulse found
 *                           2 alarming presence pulse found
 *                           3 1-Wire is shorted
 *                           5 TMTouchReset not supported
 *                          -1 TMSetup has not been run on MicroLan
 *                        -200 Invalid Session
 *                        -201 Hardware_Specific driver not found and is required
 *                                 
 */
short far pascal TMTouchReset(short port)
{
     inreg.x.ax = ((port << 4) | 0x01) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
     {
          if ((outreg.x.ax & 0x00FF) == 1) /*  device found */
          {
               if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x04 )
                  return 2;  /* alarm test performed and true*/
               else
                  return 1;
          }
          else if ( ((outreg.x.dx >> 8) & outreg.x.dx) & 0x03 )
               return 3;     /* short test */
          else
               return 0;
     }
     else
     {      
          /*  TMSetup not run on 1-Wire port */
          if ((outreg.x.ax & 0x00FF) == 1) 
             return -1;
          else
             return 5;        /* Touchreset not supported */
     }
}


/*--------------------------------------------------------------------------
 * This API call transmits 8 bits on to the 1-Wire data line and receives 
 * 8 bits concurrently. The 8 bits to transmit is a least significant byte
 * of 'outbyte' and it returns an integer where the the least significant
 * byte is the received 8 bits.  If there is an error then a negative
 * error code will be returned.  
 *
 * The return values for this API call are:
 *                                                    
 *   >=0 : LS byte is the received byte
 *    <0 : HARDWARE_SPECIFIC error 
 */
short far pascal TMTouchByte(short port, short outbyte)
{
     inreg.x.ax = (((port << 4) | 0x02) << 8) | outbyte;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

      if (!outreg.x.cflag)
        return outreg.x.ax & 0x00FF;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * This API call is used in the search rom operations.  It sends out one 
 * bit on the 1-Wire and returns one bit.  The return values are:
 *                                                    
 *   >=0 : LS bit is the received bit
 *    <0 : HARDWARE_SPECIFIC error (see TMTouchByte)
 */
short far pascal TMTouchBit(short port, short outbit)
{
     inreg.x.ax = (((port << 4) | 0x03) << 8) | outbit;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return outreg.x.ax & 0x0001;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 *  This routine sends out a 480 microsecond 12V programming pulse.  If
 *  this is for the pc COM port then there needs to be DS9097E serial brick.
 *  WARNING this function should only be called if program compatible parts 
 *  are on the 1-Wire port.  The programming pulse may damage non-program 
 *  Touch Memory. The return values are:
 *                                                    
 *     1 : program pulse sent
 *    <0 : HARDWARE_SPECIFIC error 
 *
 * (2.00) This function is new for version 2.00.
 */
short far pascal TMProgramPulse(short port)
{
     inreg.x.ax =(((port << 4) | 0x0E) << 8) | 0x01;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return 1;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}
                               
                               
/*--------------------------------------------------------------------------
 *  This API call closes a particular 1-Wire port.  After this routine is
 *  called on a 1-Wire port, the only way to use it again is to call
 *  'TMSetup'.  The return values are:
 *                                                    
 *     1 : 1-Wire port closed
 *    <0 : HARDWARE_SPECIFIC error 
 */
short far pascal TMClose(short port)
{
     inreg.x.ax =((port << 4) | 0x0F) << 8;

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     if (!outreg.x.cflag)
        return 1;
     else
        return -(short)(outreg.x.ax & 0x00FF);
}


/*-------------------------------------------------------------------------
 *  This function sets the level value of the 1-Wire.  The mode indicates 
 *  reading(1) or setting(0).  The value is as follows.
 *
 *    0 - Normal 3-5 volt 1500 ohm
 *    1 - Strong pull up capable of 50 ma at 3 volts
 *    2 - Break, low impedance zero voltage
 *    3 - Program voltage 12 volt at 10 ma suitable for EPROM programming
 *
 *  Primed(1) indicates the voltage stays the same now but get flipped 
 *  after the next TMTouchBit. Primed(2) is the same after the next 
 *  TMTouchByte.  Primed(0) means that the level change is imediate.
 *  Primed and value are not used when mode is reading(1).
 */
short far pascal TMOneWireLevel(short port, short mode, short value, short primed)
{
   inreg.x.ax = ((((uchar)(port & 0xF) << 4) | 0x0E) << 8) | 0x02;
   inreg.x.cx = ((primed & 0x0003) << 9) | ((mode & 0x0001) << 8) | (value & 0x00FF);

   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   if (!outreg.x.cflag)
      return outreg.x.cx & 0x00FF;
   else
      return -(short)(outreg.x.ax & 0x00FF);
}                                                                             


/*-------------------------------------------------------------------------
 *  This function sets the line speed characteristics of the 1-Wire.
 *  The mode indicates reading(1) or setting(0).  The value is as follows.
 *
 *    0 - Normal timing
 *    1 - Overdrive timing
 *    2 - Relaxed timing for long lines
 *
 *  When setting the mode, the new mode is returned or BNO_SUCH_FUNCTION
 *  if that value is not supported.
 */
short far pascal TMOneWireCom(short port, short mode, short value)
{
   inreg.x.ax = ((((uchar)(port & 0xF) << 4) | 0x0E) << 8) | 0x03;
   inreg.x.cx = ((mode & 0x0001) << 8) | (value & 0x00FF);

   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   if (!outreg.x.cflag)
      return outreg.x.cx & 0x00FF;
   else
      return -(short)(outreg.x.ax & 0x00FF);
}

/*--------------------------------------------------------------------------
 * This API call simply does a stream of TMTouchByte(s) of all of the 'num_tran' 
 * bytes in the 'tran_buffer' data buffer.  The values returned from the TMTouchByte(s) 
 * are placed back into the 'tran_buffer' data buffer.  Maximum number of 
 * bytes is 1023.
 *
 * This function returns:
 *                       >=0 :  length of data sent and received from MicroLan
 *
 *                        <0 : HARDWARE SPECIFIC error 
 *
 * (3.10) This function is new for version 3.10.
 */
short far pascal TMBlockStream(short port, uchar far *tran_buffer, short num_tran)
{
     inreg.x.ax = (((port << 4) | 0x0E) << 8)|(0x04);
     inreg.x.cx = num_tran;                   /* set number to write */
     inreg.x.bx = FP_OFF(tran_buffer);           /* set offset and segment */
     sreg.es = FP_SEG(tran_buffer);

     int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */

     /* check the carry flag to see if was unsuccessful */
     if (outreg.x.cflag)
        return -(short)(outreg.x.ax & 0xFF);    /* return -error code */

     return outreg.x.cx;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * The TMAdapterSpec API call, copies information about the
 * current adapter on the MicroLAN specified by 'Handle'
 * into the provided Specification structure. 
 *
 * This structure stores information about overdrive capabilities 
 * (0-no overdrive, overdrive capable), strong pullup capabilities, 
 * EPROM programming capabilities and a description of the family 
 * including  part number, alternate part number and a brief 
 * functional description.
 *
 * (3.11) This function is new for version 3.11.
 */
short far pascal TMGetAdapterSpec(short port, Specification far * fam)
{
   inreg.x.ax = (((port << 4) | 0x0E) << 8)|(0x06);
   inreg.x.bx = FP_OFF(fam);           /* set offset and segment */
   sreg.es = FP_SEG(fam);
   
   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   /* check the carry flag to see if was unsuccessful */
   if (!outreg.x.cflag)
      return outreg.x.ax & 0x00FF;
   else
      return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * Provides information about the family type of the part stored in the rom
 * buffer.
 *
 * (3.11) This function is new for version 3.11.
 */
short far pascal TMGetFamilySpec(short port, Specification far * fam)
{
   inreg.x.ax = (((port << 4) | 0x0E) << 8)|(0x05);
   inreg.x.bx = FP_OFF(fam);           /* set offset and segment */
   sreg.es = FP_SEG(fam);
   
   int86x(TMEXInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   /* check the carry flag to see if was unsuccessful */
   if (!outreg.x.cflag)
      return outreg.x.ax & 0x00FF;
   else
      return -(short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * Exportable function used to calculate the CRC8 or CRC16 of a given buffer
 * and seed Type = 0 for CRC8 and Type = 1 for CRC16
 *
 * (3.11) This function is new for version 3.11.
 */
long far pascal TMCRC(short length, uchar far *fbuf, ushort Seed, short Type)
{
   inreg.x.ax = ((0x0A) << 8)|(0x08);
   inreg.x.cx = length;                   /* set number to write */
   inreg.x.dx = ((Type & 0x0F) << 8)|(Seed & 0x00FF); 
   inreg.x.bx = FP_OFF(fbuf);           /* set offset and segment */
   sreg.es = FP_SEG(fbuf);
   
   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   /* check the carry flag to see if was unsuccessful */
   if (outreg.x.cflag)
      return -(short)(outreg.x.ax & 0xFF);    /* return -error code */
   
   return outreg.x.ax;  /* return successful number chars read */
}


/*--------------------------------------------------------------------------
 * Program a block to the status or regular memory space of a DS1982,
 * DS1985, or DS1986.  The address is in Addr in space Space (0 Regular, 1
 * status) is Bits number of bits per pass (2,4,8).  It is the job of the 
 * caller to make sure that no other devices are on the line at the same 
 * time that can not handle a programing pulse. 
 *
 * (3.11) This function is new for version 3.11.
 */ 
short far pascal TMProgramBlock(short port, uchar far *buf, short len, 
                                 short Addr, short far *Bits)
{
   inreg.x.ax = (((port << 4) | 0x0D) << 8) | (0x07);
   inreg.x.cx = Addr;                   /* set address of byte to write */
   /* bit twiddling note for dx register:  00 = 2 bits,01 = 4 bits,10 = 8 bits*/
   /* upper 2 bits of 2 is 00, of 4 is 01 of 8 is 10, so shift value 10 and and with 3*/
   inreg.x.dx = ((*Bits << 10) & 0x3000) | (len & 0xFFF); 
   inreg.x.bx = FP_OFF(buf);           /* set offset and segment */
   sreg.es = FP_SEG(buf);
   
   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   /* check the carry flag to see if was unsuccessful */
   if (outreg.x.cflag)
      return -(short)(outreg.x.ax & 0xFF);    /* return -error code */
   
   /* return the last number of bits per pass */
   *Bits = 0x02 << ((outreg.x.dx >> 12) & 0x03);
   
   return outreg.x.cx & 0xFFF; 
}


/*--------------------------------------------------------------------------
 * Search function with optional reset
 * ResetSearch = true resets the search to start again (TMFirst)
 * PerformReset = true performs reset
 * SrchCmd = one wire search command 
 *
 * (3.11) This function is new for version 3.11.
 */
short far pascal TMSearch(short port, short ResetSearch, 
                short PerformReset, short SrchCmd)
{
   inreg.x.ax = (((port << 4) | 0x0D) << 8) | (0x06);
   inreg.x.cx = ((ResetSearch & 0x01) << 8) | ((PerformReset & 0x01) << 9) | 
                 (SrchCmd & 0xFF);
   
   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   /* check the carry flag to see if was unsuccessful */
   if (!outreg.x.cflag)
      return outreg.x.ax & 0x00FF;
   else
      return (short)(outreg.x.ax & 0x00FF);
}


/*--------------------------------------------------------------------------
 * Read the default port and type from the Registry
 *
 * (3.11) This function is new for version 3.11.
 */
short far pascal TMReadDefaultPort(short far *portnum, short far *porttype)
{
   inreg.x.ax = ((0x0E) << 8)|(0x05);
   
   int86x(DOWInt,&inreg,&outreg,&sreg);  /* call the interrupt */
   
   if (outreg.x.cflag)
      return -(short)(outreg.x.ax & 0xFF);    /* return -error code */
   
   /* set the port type an number */
   *portnum = outreg.x.cx >> 8;
   *porttype = outreg.x.cx & 0xFF;

   return 1; 
}


/*--------------------------------------------------------------------------
 * Search for the extended 1-Wire functions interrupt 'DOWInt' and the
 * TMEX interrupt 'TMEXInt'.  Return TRUE if both found and FALSE otherwise.
 * Note that this function is looking for at least 'DOWF' and 'TMEXF'.
 * This mandates at least version 1.10 of the drivers.
 */
short far pascal FindInts(short far *DOWInt, short far *TMEXInt)
{
     short i;
     char idstr[51];

     /* initialize the interrupt number */
     *DOWInt = 0;
     *TMEXInt = 0;

     /* loop to search each interrupt vector for Extend and TMEX */
     for (i = 0x60; i <= 0x66; i++)
     {
          /* get the id string for i'th vector */
          if (GetVectID(i,idstr) != 1)
             continue;    /* skip this interrupt if no id */

          /* check to see if this is a tmex interrupt */
          if (strncmp(idstr,"TMEX",4) == 0 && idstr[4] == 'F')
             *TMEXInt = i;

          /* check to see if this is a Extended 1-Wire function int */
          if (strncmp(idstr,"DOW",3) == 0 && idstr[3] == 'F' && idstr[6] != 'X')
             *DOWInt = i;
     }

     /* check to see if both interrupt were found */
     if (*DOWInt && *TMEXInt)
        return 1;             /* both found */
     else
        return 0;            /* both were not found */
}


/*--------------------------------------------------------------------------
 * Check an interrupt and copy its id string into the string "idstr".
 *
 *  returns  1 id string found and placed in 'idstr'
 *           0 vector is zero
 *          -1 vector string is larger then 50 characters
 *
 */
short far pascal GetVectID(short TInt, char far *idstr)
{
     uchar far *peekidstr;
     short i,len;

     // look at vector to see if I'm already there or is empty
     peekidstr = (uchar far *)(*((unsigned long far *)(4*TInt)));

     if (!peekidstr)  /* vector is zero */
        return 0;

     len = peekidstr[5];

     /* check to see if length too large */
     if (len > 50)
        return -1;

     /* copy idstring over to 'idstr' */
     for (i = 0; i < len; i++)
         idstr[i] = peekidstr[i+6];

     idstr[i] = 0;  /* zero terminate the idstring */

     return 1;  /* return string found */
}

